@@ -8,9 +8,18 @@ extern "C" {
 #include <stdlib.h>
 #include <stdarg.h>
 #include <stdint.h>
+#include <stdbool.h>
 
 #include <sys/time.h>
 
+/* a tri-state yes/no/don't-know type */
+
+typedef enum {
+  TICKIT_NO    =  0,
+  TICKIT_YES   =  1,
+  TICKIT_MAYBE = -1,
+} TickitMaybeBool;
+
 /*
  * Tickit events
  */
@@ -88,29 +97,29 @@ TickitPen *tickit_pen_new_attrs(TickitPenAttr attr, ...);
 TickitPen *tickit_pen_clone(const TickitPen *orig);
 void       tickit_pen_destroy(TickitPen *pen);
 
-int tickit_pen_has_attr(const TickitPen *pen, TickitPenAttr attr);
-int tickit_pen_is_nonempty(const TickitPen *pen);
-int tickit_pen_nondefault_attr(const TickitPen *pen, TickitPenAttr attr);
-int tickit_pen_is_nondefault(const TickitPen *pen);
+bool tickit_pen_has_attr(const TickitPen *pen, TickitPenAttr attr);
+bool tickit_pen_is_nonempty(const TickitPen *pen);
+bool tickit_pen_nondefault_attr(const TickitPen *pen, TickitPenAttr attr);
+bool tickit_pen_is_nondefault(const TickitPen *pen);
 
-int  tickit_pen_get_bool_attr(const TickitPen *pen, TickitPenAttr attr);
-void tickit_pen_set_bool_attr(TickitPen *pen, TickitPenAttr attr, int val);
+bool tickit_pen_get_bool_attr(const TickitPen *pen, TickitPenAttr attr);
+void tickit_pen_set_bool_attr(TickitPen *pen, TickitPenAttr attr, bool val);
 
 int  tickit_pen_get_int_attr(const TickitPen *pen, TickitPenAttr attr);
 void tickit_pen_set_int_attr(TickitPen *pen, TickitPenAttr attr, int val);
 
 int  tickit_pen_get_colour_attr(const TickitPen *pen, TickitPenAttr attr);
 void tickit_pen_set_colour_attr(TickitPen *pen, TickitPenAttr attr, int value);
-int  tickit_pen_set_colour_attr_desc(TickitPen *pen, TickitPenAttr attr, const char *value);
+bool tickit_pen_set_colour_attr_desc(TickitPen *pen, TickitPenAttr attr, const char *value);
 
 void tickit_pen_clear_attr(TickitPen *pen, TickitPenAttr attr);
 void tickit_pen_clear(TickitPen *pen);
 
-int tickit_pen_equiv_attr(const TickitPen *a, const TickitPen *b, TickitPenAttr attr);
-int tickit_pen_equiv(const TickitPen *a, const TickitPen *b);
+bool tickit_pen_equiv_attr(const TickitPen *a, const TickitPen *b, TickitPenAttr attr);
+bool tickit_pen_equiv(const TickitPen *a, const TickitPen *b);
 
 void tickit_pen_copy_attr(TickitPen *dst, const TickitPen *src, TickitPenAttr attr);
-void tickit_pen_copy(TickitPen *dst, const TickitPen *src, int overwrite);
+void tickit_pen_copy(TickitPen *dst, const TickitPen *src, bool overwrite);
 
 typedef void TickitPenEventFn(TickitPen *tt, TickitEventType ev, TickitEvent *args, void *data);
 
@@ -141,10 +150,10 @@ static inline int tickit_rect_bottom(const TickitRect *rect)
 static inline int tickit_rect_right (const TickitRect *rect)
 { return rect->left + rect->cols; }
 
-int tickit_rect_intersect(TickitRect *dst, const TickitRect *a, const TickitRect *b);
+bool tickit_rect_intersect(TickitRect *dst, const TickitRect *a, const TickitRect *b);
 
-int tickit_rect_intersects(const TickitRect *a, const TickitRect *b);
-int tickit_rect_contains(const TickitRect *large, const TickitRect *small);
+bool tickit_rect_intersects(const TickitRect *a, const TickitRect *b);
+bool tickit_rect_contains(const TickitRect *large, const TickitRect *small);
 
 int tickit_rect_add(TickitRect ret[3], const TickitRect *a, const TickitRect *b);
 int tickit_rect_subtract(TickitRect ret[4], const TickitRect *orig, const TickitRect *hole);
@@ -166,8 +175,8 @@ size_t tickit_rectset_get_rects(const TickitRectSet *trs, TickitRect rects[], si
 void tickit_rectset_add(TickitRectSet *trs, const TickitRect *rect);
 void tickit_rectset_subtract(TickitRectSet *trs, const TickitRect *rect);
 
-int tickit_rectset_intersects(const TickitRectSet *trs, const TickitRect *rect);
-int tickit_rectset_contains(const TickitRectSet *trs, const TickitRect *rect);
+bool tickit_rectset_intersects(const TickitRectSet *trs, const TickitRect *rect);
+bool tickit_rectset_contains(const TickitRectSet *trs, const TickitRect *rect);
 
 /*
  * TickitTerm
@@ -187,20 +196,27 @@ int  tickit_term_get_output_fd(const TickitTerm *tt);
 void tickit_term_set_output_func(TickitTerm *tt, TickitTermOutputFunc *fn, void *user);
 void tickit_term_set_output_buffer(TickitTerm *tt, size_t len);
 
-void tickit_term_await_started(TickitTerm *tt, const struct timeval *timeout);
+// deprecate the unitless version
+#define tickit_term_await_started(tt, timeout) tickit_term_await_started_tv(tt, timeout)
+void tickit_term_await_started_msec(TickitTerm *tt, long msec);
+void tickit_term_await_started_tv(TickitTerm *tt, const struct timeval *timeout);
 void tickit_term_flush(TickitTerm *tt);
 
 /* fd is allowed to be unset (-1); works abstractly */
 void tickit_term_set_input_fd(TickitTerm *tt, int fd);
 int  tickit_term_get_input_fd(const TickitTerm *tt);
 
-int  tickit_term_get_utf8(const TickitTerm *tt);
-void tickit_term_set_utf8(TickitTerm *tt, int utf8);
+TickitMaybeBool tickit_term_get_utf8(const TickitTerm *tt);
+void tickit_term_set_utf8(TickitTerm *tt, bool utf8);
 
 void tickit_term_input_push_bytes(TickitTerm *tt, const char *bytes, size_t len);
 void tickit_term_input_readable(TickitTerm *tt);
-int  tickit_term_input_check_timeout(TickitTerm *tt);
-void tickit_term_input_wait(TickitTerm *tt, const struct timeval *timeout);
+// deprecate the unitless versions
+#define tickit_term_input_check_timeout(tt) tickit_term_input_check_timeout_msec(tt)
+#define tickit_term_input_wait(tt, timeout) tickit_term_input_wait_tv(tt, timeout)
+int  tickit_term_input_check_timeout_msec(TickitTerm *tt);
+void tickit_term_input_wait_msec(TickitTerm *tt, long msec);
+void tickit_term_input_wait_tv(TickitTerm *tt, const struct timeval *timeout);
 
 void tickit_term_get_size(const TickitTerm *tt, int *lines, int *cols);
 void tickit_term_set_size(TickitTerm *tt, int lines, int cols);
@@ -215,15 +231,15 @@ void tickit_term_print(TickitTerm *tt, const char *str);
 void tickit_term_printn(TickitTerm *tt, const char *str, size_t len);
 void tickit_term_printf(TickitTerm *tt, const char *fmt, ...);
 void tickit_term_vprintf(TickitTerm *tt, const char *fmt, va_list args);
-int  tickit_term_goto(TickitTerm *tt, int line, int col);
+bool tickit_term_goto(TickitTerm *tt, int line, int col);
 void tickit_term_move(TickitTerm *tt, int downward, int rightward);
-int  tickit_term_scrollrect(TickitTerm *tt, int top, int left, int lines, int cols, int downward, int rightward);
+bool tickit_term_scrollrect(TickitTerm *tt, TickitRect rect, int downward, int rightward);
 
 void tickit_term_chpen(TickitTerm *tt, const TickitPen *pen);
 void tickit_term_setpen(TickitTerm *tt, const TickitPen *pen);
 
 void tickit_term_clear(TickitTerm *tt);
-void tickit_term_erasech(TickitTerm *tt, int count, int moveend);
+void tickit_term_erasech(TickitTerm *tt, int count, TickitMaybeBool moveend);
 
 typedef enum {
   /* This is part of the API so additions must go at the end only */
@@ -252,9 +268,9 @@ typedef enum {
   TICKIT_TERM_CURSORSHAPE_LEFT_BAR,
 } TickitTermCursorShape;
 
-int tickit_term_getctl_int(TickitTerm *tt, TickitTermCtl ctl, int *value);
-int tickit_term_setctl_int(TickitTerm *tt, TickitTermCtl ctl, int value);
-int tickit_term_setctl_str(TickitTerm *tt, TickitTermCtl ctl, const char *value);
+bool tickit_term_getctl_int(TickitTerm *tt, TickitTermCtl ctl, int *value);
+bool tickit_term_setctl_int(TickitTerm *tt, TickitTermCtl ctl, int value);
+bool tickit_term_setctl_str(TickitTerm *tt, TickitTermCtl ctl, const char *value);
 
 /*
  * String handling utilities
@@ -331,7 +347,7 @@ void tickit_renderbuffer_translate(TickitRenderBuffer *rb, int downward, int rig
 void tickit_renderbuffer_clip(TickitRenderBuffer *rb, TickitRect *rect);
 void tickit_renderbuffer_mask(TickitRenderBuffer *rb, TickitRect *mask);
 
-int tickit_renderbuffer_has_cursorpos(const TickitRenderBuffer *rb);
+bool tickit_renderbuffer_has_cursorpos(const TickitRenderBuffer *rb);
 void tickit_renderbuffer_get_cursorpos(const TickitRenderBuffer *rb, int *line, int *col);
 void tickit_renderbuffer_goto(TickitRenderBuffer *rb, int line, int col);
 void tickit_renderbuffer_ungoto(TickitRenderBuffer *rb);
@@ -344,13 +360,15 @@ void tickit_renderbuffer_save(TickitRenderBuffer *rb);
 void tickit_renderbuffer_savepen(TickitRenderBuffer *rb);
 void tickit_renderbuffer_restore(TickitRenderBuffer *rb);
 
-void tickit_renderbuffer_skip_at(TickitRenderBuffer *rb, int line, int col, int len);
-void tickit_renderbuffer_skip(TickitRenderBuffer *rb, int len);
+void tickit_renderbuffer_skip_at(TickitRenderBuffer *rb, int line, int col, int cols);
+void tickit_renderbuffer_skip(TickitRenderBuffer *rb, int cols);
 void tickit_renderbuffer_skip_to(TickitRenderBuffer *rb, int col);
 int tickit_renderbuffer_text_at(TickitRenderBuffer *rb, int line, int col, char *text, TickitPen *pen);
+int tickit_renderbuffer_textn_at(TickitRenderBuffer *rb, int line, int col, char *text, size_t len, TickitPen *pen);
 int tickit_renderbuffer_text(TickitRenderBuffer *rb, char *text, TickitPen *pen);
-void tickit_renderbuffer_erase_at(TickitRenderBuffer *rb, int line, int col, int len, TickitPen *pen);
-void tickit_renderbuffer_erase(TickitRenderBuffer *rb, int len, TickitPen *pen);
+int tickit_renderbuffer_textn(TickitRenderBuffer *rb, char *text, size_t len, TickitPen *pen);
+void tickit_renderbuffer_erase_at(TickitRenderBuffer *rb, int line, int col, int cols, TickitPen *pen);
+void tickit_renderbuffer_erase(TickitRenderBuffer *rb, int cols, TickitPen *pen);
 void tickit_renderbuffer_erase_to(TickitRenderBuffer *rb, int col, TickitPen *pen);
 void tickit_renderbuffer_eraserect(TickitRenderBuffer *rb, TickitRect *rect, TickitPen *pen);
 void tickit_renderbuffer_clear(TickitRenderBuffer *rb, TickitPen *pen);
@@ -376,6 +394,8 @@ void tickit_renderbuffer_vline_at(TickitRenderBuffer *rb, int startline, int end
 
 void tickit_renderbuffer_flush_to_term(TickitRenderBuffer *rb, TickitTerm *tt);
 
+void tickit_renderbuffer_blit(TickitRenderBuffer *dst, TickitRenderBuffer *src);
+
 // This API is still somewhat experimental
 
 typedef struct {
@@ -393,7 +413,7 @@ TickitRenderBufferLineMask tickit_renderbuffer_get_cell_linemask(TickitRenderBuf
 TickitPen *tickit_renderbuffer_get_cell_pen(TickitRenderBuffer *rb, int line, int col);
 
 struct TickitRenderBufferSpanInfo {
-  int is_active;
+  bool is_active;
   int n_columns;
   char *text;
   size_t len;
