@@ -10,7 +10,7 @@ use warnings;
 use feature qw( switch );
 no if $] >= 5.017011, warnings => 'experimental::smartmatch';
 
-our $VERSION = '0.49';
+our $VERSION = '0.50';
 
 use Carp;
 use Scalar::Util qw( refaddr );
@@ -581,69 +581,9 @@ sub get_cell
    }
 }
 
-=head2 $rb->flush_to_window( $win )
-
-Renders the stored content to the given L<Tickit::Window>. After this, the
-buffer will be cleared and reset back to initial state.
-
-Flushing to the window is now deprecated. Widgets should render to the
-renderbuffer given to their C<render_to_rb> method. Applications that wish to
-perform lower-level drawing without Widgets or Windows should construct a
-renderbuffer and use the C<flush_to_term> method.
-
-=head2 $rb->flush_to_term( $term )
-
-Renders the stored content to the given L<Tickit::Term>. After this, the
-buffer will be cleared and reset back to initial state.
-
-=cut
-
-my $warned_flush_to_window;
-
 sub flush_to_window
 {
-   my $self = shift;
-   my ( $target ) = @_;
-
-   $warned_flush_to_window or
-      $warned_flush_to_window++, warnings::warnif
-         deprecated => "Tickit::RenderBuffer->flush_to_window is deprecated";
-
-   foreach my $line ( 0 .. $self->lines-1 ) {
-      my $phycol;
-
-      for ( my $col = 0; $col < $self->cols ; ) {
-         my ( $len, $text, $pen ) = $self->_xs_get_span( $line, $col );
-
-         $col += $len, next if !defined $text; # SKIP
-
-         if( !defined $phycol or $phycol < $col ) {
-            $target->goto( $line, $col );
-         }
-         $phycol = $col;
-
-         if( length $text ) {
-            # TEXT, LINE, CHAR
-            $target->print( $text, $pen );
-            $phycol += $len;
-         }
-         else {
-            # ERASE
-            # No need to set moveend=true to erasech unless we actually
-            # have more content;
-            my $moveend = $col + $len < $self->cols &&
-                          defined +( $self->_xs_get_span( $line, $col + $len ) )[1];
-
-            $target->erasech( $len, $moveend || undef, $pen );
-            $phycol += $len;
-            undef $phycol unless $moveend;
-         }
-
-         $col += $len;
-      }
-   }
-
-   $self->reset;
+   croak "Tickit::RenderBuffer->flush_to_window has been removed";
 }
 
 =head2 $rb->flush_to_term( $term )
