@@ -8,7 +8,7 @@ package Tickit::Widget;
 use strict;
 use warnings;
 
-our $VERSION = '0.49';
+our $VERSION = '0.50';
 
 use Carp;
 use Scalar::Util qw( weaken );
@@ -696,7 +696,7 @@ again, then calls C<set_requested_size>.
 sub resized
 {
    my $self = shift;
-   # 'scalar' just in case of odd behavious in subclasses
+   # 'scalar' just in case of odd behaviour in subclasses
    $self->set_requested_size( scalar $self->lines, scalar $self->cols );
 }
 
@@ -777,24 +777,6 @@ sub redraw
    $self->window->expose;
 }
 
-sub _do_clear
-{
-   my $self = shift;
-   my ( $rect ) = @_;
-   my $window = $self->window or return;
-
-   if( my $parentwin = $window->parent ) {
-      my $bg       = $window->get_effective_penattr( 'bg' );
-      my $parentbg = $parentwin->get_effective_penattr( 'bg' );
-
-      return 0 if !defined $bg and !defined $parentbg;
-      return 0 if  defined $bg and  defined $parentbg and $bg == $parentbg;
-   }
-
-   $window->clearrect( $rect );
-   return 1;
-}
-
 =head2 $pen = $widget->pen
 
 Returns the widget's L<Tickit::Pen>. Modifying an attribute of the returned
