@@ -9,7 +9,7 @@ use strict;
 use warnings;
 use 5.010; # //
 
-our $VERSION = '0.49';
+our $VERSION = '0.50';
 
 use Carp;
 
@@ -943,8 +943,6 @@ sub _do_expose
    if( my $on_expose = $self->{on_expose} ) {
       $rb->save;
 
-      local $self->{exposure_rb} = $rb;
-
       $on_expose->( $self, $rb, $rect );
 
       $rb->restore;
@@ -1327,274 +1325,24 @@ sub _get_span_visibility
    return ( $vis, $len );
 }
 
-=head2 $win->goto( $line, $col )
-
-Moves the cursor to the given position within the window. Both C<$line> and
-C<$col> are 0-based. The given position does not have to lie within the bounds
-of the window. Lines above or below the window are never displayed, columns to
-the left or right are clipped as appropriate. The virtual position of the
-cursor is still tracked even if it is not visibly displayed on the actual
-terminal.
-
-Direct drawing to the window is now entirely deprecated; widgets and
-applications should instead use the render buffer passed to the C<on_expose>
-event.
-
-=cut
-
-my $warned_direct_draw;
-
 sub goto
 {
-   my $win = shift;
-   my ( $line, $col ) = @_;
-
-   $warned_direct_draw or
-      $warned_direct_draw++, warnings::warnif
-         deprecated => "Tickit::Window direct drawing methods are deprecated";
-
-   if( my $rb = $win->{exposure_rb} ) {
-      $rb->goto( $line, $col );
-      return;
-   }
-
-   $win->{output_line}   = $line;
-   $win->{output_column} = $col;
-   $win->{output_needs_goto} = 0;
-
-   my ( $vis ) = $win->_get_span_visibility( $line, $col );
-   return unless $vis;
-
-   while( $win ) {
-      return if $line < 0 or $line >= $win->lines;
-      return if $col  < 0 or $col  >= $win->cols;
-
-      return unless $win->{visible};
-
-      $line += $win->top;
-      $col  += $win->left;
-
-      my $parent = $win->parent or last;
-      $win = $parent;
-   }
-
-   $win->term->goto( $line, $col );
-   $win->_needs_flush;
+   croak "Tickit::Window direct drawing methods have been removed; use the RenderBuffer instead";
 }
 
-=head2 $pos = $win->print( $text, $pen )
-
-=head2 $pos = $win->print( $text, %attrs )
-
-Print the given text to the terminal at the current cursor position using the
-pen of the window, possibly overridden by any extra attributes in the given
-C<Tickit::Pen> instance, or directly in the given hash, if one is provided.
-
-Returns a L<Tickit::StringPos> object giving the total count of string
-printed, including in obscured sections covered by other windows, or clipped
-by window boundaries.
-
-Direct drawing to the window is now entirely deprecated; widgets and
-applications should instead use the render buffer passed to the C<on_expose>
-event.
-
-=cut
-
 sub print
 {
-   my $self = shift;
-   my $text = shift;
-
-   $warned_direct_draw or
-      $warned_direct_draw++, warnings::warnif
-         deprecated => "Tickit::Window direct drawing methods are deprecated";
-
-   my $pen = ( @_ == 1 ) ? shift->as_mutable : Tickit::Pen::Mutable->new( @_ );
-
-   if( my $rb = $self->{exposure_rb} ) {
-      return $rb->text( $text, $pen );
-   }
-
-   # First collect up the pen attributes and abort early if any window is
-   # invisible
-   for( my $win = $self; $win; $win = $win->parent ) {
-      return unless $win->{visible};
-      $pen->default_from( $win->pen );
-   }
-
-   my $line = $self->{output_line};
-   my $term = $self->term;
-
-   my $need_goto = $self->{output_needs_goto};
-   my ( $abs_top, $abs_left );
-   my $need_flush = 0;
-
-   my $pos = Tickit::StringPos->zero;
-
-   my $total_len = length $text;
-
-   while( $pos->codepoints < $total_len ) {
-      my ( $vis, $cols ) = $self->_get_span_visibility( $line, $self->{output_column} + $pos->columns );
-
-      if( !$vis and !defined $cols ) {
-         string_countmore( $text, $pos );
-         last;
-      }
-
-      my $prev_cp  = $pos->codepoints;
-      my $prev_col = $pos->columns;
-      defined string_countmore( $text, $pos, Tickit::StringPos->limit_columns( $cols + $pos->columns ) ) or
-         croak "Encountered non-Unicode text in ->print; bailing out";
-
-      # TODO: This would be more efficient in bytes, but 'use bytes' breaks
-      # the UTF-8ness of the return value
-      my $chunk = substr $text, $prev_cp, $pos->codepoints - $prev_cp;
-
-      if( $vis ) {
-         if( $need_goto ) {
-            $abs_top  //= $self->abs_top;
-            $abs_left //= $self->abs_left;
-            $term->goto( $abs_top + $line, $abs_left + $self->{output_column} + $prev_col );
-            $need_goto = 0;
-         }
-
-         $term->setpen( $pen );
-         $term->print( $chunk );
-
-         $need_flush = 1;
-      }
-      else {
-         # TODO: $term->move( undef, $pos->columns - $prev_col )
-         $need_goto = 1;
-      }
-   }
-
-   $self->{output_column} += $pos->columns;
-
-   $self->root->_needs_flush if $need_flush;
-   $self->{output_needs_goto} = $need_goto;
-
-   return $pos;
+   croak "Tickit::Window direct drawing methods have been removed; use the RenderBuffer instead";
 }
 
-=head2 $pos = $win->erasech( $count, $moveend, $pen )
-
-=head2 $pos = $win->erasech( $count, $moveend, %attrs )
-
-Erase C<$count> columns forwards. If C<$moveend> is true, the cursor will be
-placed at the end of the erased region. If defined but false, it will not move
-from its current location. If undefined, the terminal will take which ever
-option it can implement most efficiently.
-
-If a C<Tickit::Pen> or pen attributes are provided, they are used to override
-the background colour for the erased region.
-
-Returns a L<Tickit::StringPos> object giving the total count of string
-printed, including in obscured sections covered by other windows, or clipped
-by window boundaries. Only the C<columns> field will be valid; the others will
-be C<-1>.
-
-Direct drawing to the window is now entirely deprecated; widgets and
-applications should instead use the render buffer passed to the C<on_expose>
-event.
-
-=cut
-
 sub erasech
 {
-   my $self = shift;
-   my $count = shift;
-   my $moveend = shift;
-
-   $warned_direct_draw or
-      $warned_direct_draw++, warnings::warnif
-         deprecated => "Tickit::Window direct drawing methods are deprecated";
-
-   my $pen = ( @_ == 1 ) ? shift->as_mutable : Tickit::Pen::Mutable->new( @_ );
-
-   if( my $rb = $self->{exposure_rb} ) {
-      return $rb->erase( $count, $pen );
-   }
-
-   # First collect up the pen attributes and abort early if any window is
-   # invisible
-   for( my $win = $self; $win; $win = $win->parent ) {
-      return unless $win->{visible};
-      $pen->default_from( $win->pen );
-   }
-
-   my $line = $self->{output_line};
-   my $term = $self->term;
-
-   my $need_goto = $self->{output_needs_goto};
-   my ( $abs_top, $abs_left );
-   my $need_flush = 0;
-
-   my $orig_count = $count;
-   while( $count ) {
-      my ( $vis, $len ) = $self->_get_span_visibility( $line, $self->{output_column} );
-
-      last if !$vis and !defined $len;
-
-      $len = $count if $len > $count;
-
-      if( $vis ) {
-         if( $need_goto ) {
-            $abs_top  //= $self->abs_top;
-            $abs_left //= $self->abs_left;
-            $term->goto( $abs_top + $line, $abs_left + $self->{output_column} );
-            $need_goto = 0;
-         }
-
-         $term->setpen( $pen );
-         $term->erasech( $len, $moveend );
-
-         $need_flush = 1;
-      }
-      else {
-         $need_goto = 1;
-      }
-
-      $self->{output_column} += $len;
-      $count -= $len;
-   }
-
-   $self->root->_needs_flush if $need_flush;
-   $self->{output_needs_goto} = $need_goto;
-
-   return if !defined wantarray;
-   return Tickit::StringPos->limit_columns( $orig_count );
+   croak "Tickit::Window direct drawing methods have been removed; use the RenderBuffer instead";
 }
 
-=head2 $win->clearrect( $rect )
-
-=head2 $win->clearrect( $rect, $pen )
-
-=head2 $win->clearrect( $rect, %attrs )
-
-Erase the content of the window within the given L<Tickit::Rect>. If a
-C<Tickit::Pen> or pen attributes are provided, they are used to override the
-background colour for the erased region.
-
-=cut
-
 sub clearrect
 {
-   my $self = shift;
-   my $rect = shift;
-
-   my $pen = ( @_ == 1 ) ? shift->as_mutable : Tickit::Pen::Mutable->new( @_ );
-
-   if( $rect->top == 0 and $rect->left == 0 and
-       $rect->bottom == $self->lines and $rect->right == $self->cols ) {
-      $self->clear;
-      return;
-   }
-
-   foreach my $line ( $rect->linerange ) {
-      $self->goto( $line, $rect->left );
-      $self->erasech( $rect->cols, undef, $pen );
-   }
+   croak "Tickit::Window direct drawing methods have been removed; use the RenderBuffer instead";
 }
 
 =head2 $success = $win->scrollrect( $rect, $downward, $rightward )
