@@ -44,7 +44,7 @@ struct TickitTerm {
   struct timeval        input_timeout_at; /* absolute time */
 
   const char *termtype;
-  signed char is_utf8;  /* -1 == unknown */
+  TickitMaybeBool is_utf8;
 
   char *outbuffer;
   size_t outbuffer_len; /* size of outbuffer */
@@ -72,9 +72,9 @@ static TermKey *get_termkey(TickitTerm *tt)
 {
   if(!tt->termkey) {
     int flags = 0;
-    if(tt->is_utf8 > 0)
+    if(tt->is_utf8 == TICKIT_YES)
       flags |= TERMKEY_FLAG_UTF8;
-    else if(tt->is_utf8 == 0)
+    else if(tt->is_utf8 == TICKIT_NO)
       flags |= TERMKEY_FLAG_RAW;
 
     tt->termkey = termkey_new(tt->infd, TERMKEY_FLAG_EINTR | flags);
@@ -138,7 +138,7 @@ TickitTerm *tickit_term_new_for_driver(TickitTermDriver *ttd)
   tt->tmpbuffer = NULL;
   tt->tmpbuffer_len = 0;
 
-  tt->is_utf8 = -1;
+  tt->is_utf8 = TICKIT_MAYBE;
 
   /* Initially; the driver may provide a more accurate value */
   tt->lines = 25;
@@ -179,6 +179,8 @@ void tickit_term_free(TickitTerm *tt)
     (*tt->driver->vtable->destroy)(tt->driver);
   }
 
+  tickit_term_flush(tt);
+
   if(tt->termkey)
     termkey_destroy(tt->termkey);
 
@@ -311,12 +313,12 @@ int tickit_term_get_input_fd(const TickitTerm *tt)
   return tt->infd;
 }
 
-int tickit_term_get_utf8(const TickitTerm *tt)
+TickitMaybeBool tickit_term_get_utf8(const TickitTerm *tt)
 {
   return tt->is_utf8;
 }
 
-void tickit_term_set_utf8(TickitTerm *tt, int utf8)
+void tickit_term_set_utf8(TickitTerm *tt, bool utf8)
 {
   tt->is_utf8 = !!utf8;
 
@@ -334,7 +336,26 @@ void tickit_term_set_utf8(TickitTerm *tt, int utf8)
   }
 }
 
+// provide link-time legacy wrappers
+#undef tickit_term_await_started
+
 void tickit_term_await_started(TickitTerm *tt, const struct timeval *timeout)
+{
+  tickit_term_await_started_tv(tt, timeout);
+}
+
+void tickit_term_await_started_msec(TickitTerm *tt, long msec)
+{
+  if(msec > -1)
+    tickit_term_await_started_tv(tt, &(struct timeval){
+        .tv_sec  = msec / 1000,
+        .tv_usec = (msec % 1000) * 1000,
+    });
+  else
+    tickit_term_await_started_tv(tt, NULL);
+}
+
+void tickit_term_await_started_tv(TickitTerm *tt, const struct timeval *timeout)
 {
   if(tt->state == STARTED)
     return;
@@ -373,7 +394,7 @@ void tickit_term_await_started(TickitTerm *tt, const struct timeval *timeout)
     if(timeout.tv_sec < 0)
       break;
 
-    tickit_term_input_wait(tt, &timeout);
+    tickit_term_input_wait_tv(tt, &timeout);
   }
 
   tt->state = STARTED;
@@ -474,7 +495,16 @@ void tickit_term_input_readable(TickitTerm *tt)
   get_keys(tt, tk);
 }
 
+// provide link-time legacy wrappers
+#undef tickit_term_input_check_timeout
+#undef tickit_term_input_wait
+
 int tickit_term_input_check_timeout(TickitTerm *tt)
+{
+  return tickit_term_input_check_timeout_msec(tt);
+}
+
+int tickit_term_input_check_timeout_msec(TickitTerm *tt)
 {
   if(tt->input_timeout_at.tv_sec == -1)
     return -1;
@@ -508,18 +538,25 @@ int tickit_term_input_check_timeout(TickitTerm *tt)
 }
 
 void tickit_term_input_wait(TickitTerm *tt, const struct timeval *timeout)
+{
+  tickit_term_input_wait_tv(tt, timeout);
+}
+
+void tickit_term_input_wait_msec(TickitTerm *tt, long msec)
 {
   TermKey *tk = get_termkey(tt);
   TermKeyKey key;
 
-  struct timeval to_copy;
-  if(timeout)
-    to_copy = *timeout;
+  struct timeval timeout;
+  if(msec > -1) {
+    timeout.tv_sec = msec / 1000;
+    timeout.tv_usec = (msec % 1000) * 1000;
+  }
 
   fd_set readfds;
   int fd = termkey_get_fd(tk);
   FD_SET(fd, &readfds);
-  if(select(fd + 1, &readfds, NULL, NULL, timeout ? &to_copy : NULL) == 1) {
+  if(select(fd + 1, &readfds, NULL, NULL, msec > -1 ? &timeout : NULL) == 1) {
     termkey_advisereadable(tk);
   }
 
@@ -529,6 +566,14 @@ void tickit_term_input_wait(TickitTerm *tt, const struct timeval *timeout)
   }
 }
 
+void tickit_term_input_wait_tv(TickitTerm *tt, const struct timeval *timeout)
+{
+  if(timeout)
+    tickit_term_input_wait_msec(tt, (long)(timeout->tv_sec) + (timeout->tv_usec / 1000));
+  else
+    tickit_term_input_wait_msec(tt, -1);
+}
+
 void tickit_term_flush(TickitTerm *tt)
 {
   if(tt->outbuffer_cur == 0)
@@ -630,7 +675,7 @@ void tickit_term_vprintf(TickitTerm *tt, const char *fmt, va_list args)
   va_end(args2);
 }
 
-int tickit_term_goto(TickitTerm *tt, int line, int col)
+bool tickit_term_goto(TickitTerm *tt, int line, int col)
 {
   return (*tt->driver->vtable->goto_abs)(tt->driver, line, col);
 }
@@ -640,14 +685,8 @@ void tickit_term_move(TickitTerm *tt, int downward, int rightward)
   (*tt->driver->vtable->move_rel)(tt->driver, downward, rightward);
 }
 
-int tickit_term_scrollrect(TickitTerm *tt, int top, int left, int lines, int cols, int downward, int rightward)
+bool tickit_term_scrollrect(TickitTerm *tt, TickitRect rect, int downward, int rightward)
 {
-  TickitRect rect = {
-    .top   = top,
-    .left  = left,
-    .lines = lines,
-    .cols  = cols,
-  };
   return (*tt->driver->vtable->scrollrect)(tt->driver, &rect, downward, rightward);
 }
 
@@ -725,22 +764,22 @@ void tickit_term_clear(TickitTerm *tt)
   (*tt->driver->vtable->clear)(tt->driver);
 }
 
-void tickit_term_erasech(TickitTerm *tt, int count, int moveend)
+void tickit_term_erasech(TickitTerm *tt, int count, TickitMaybeBool moveend)
 {
   (*tt->driver->vtable->erasech)(tt->driver, count, moveend);
 }
 
-int tickit_term_getctl_int(TickitTerm *tt, TickitTermCtl ctl, int *value)
+bool tickit_term_getctl_int(TickitTerm *tt, TickitTermCtl ctl, int *value)
 {
   return (*tt->driver->vtable->getctl_int)(tt->driver, ctl, value);
 }
 
-int tickit_term_setctl_int(TickitTerm *tt, TickitTermCtl ctl, int value)
+bool tickit_term_setctl_int(TickitTerm *tt, TickitTermCtl ctl, int value)
 {
   return (*tt->driver->vtable->setctl_int)(tt->driver, ctl, value);
 }
 
-int tickit_term_setctl_str(TickitTerm *tt, TickitTermCtl ctl, const char *value)
+bool tickit_term_setctl_str(TickitTerm *tt, TickitTermCtl ctl, const char *value)
 {
   return (*tt->driver->vtable->setctl_str)(tt->driver, ctl, value);
 }
