@@ -34,12 +34,13 @@ struct XTermDriver {
   } initialised;
 };
 
-static void print(TickitTermDriver *ttd, const char *str, size_t len)
+static bool print(TickitTermDriver *ttd, const char *str, size_t len)
 {
   tickit_termdrv_write_str(ttd, str, len);
+  return true;
 }
 
-static int goto_abs(TickitTermDriver *ttd, int line, int col)
+static bool goto_abs(TickitTermDriver *ttd, int line, int col)
 {
   if(line != -1 && col > 0)
     tickit_termdrv_write_strf(ttd, "\e[%d;%dH", line+1, col+1);
@@ -52,10 +53,10 @@ static int goto_abs(TickitTermDriver *ttd, int line, int col)
   else if(col != -1)
     tickit_termdrv_write_str(ttd, "\e[G", 3);
 
-  return 1;
+  return true;
 }
 
-static void move_rel(TickitTermDriver *ttd, int downward, int rightward)
+static bool move_rel(TickitTermDriver *ttd, int downward, int rightward)
 {
   if(downward > 1)
     tickit_termdrv_write_strf(ttd, "\e[%dB", downward);
@@ -74,14 +75,16 @@ static void move_rel(TickitTermDriver *ttd, int downward, int rightward)
     tickit_termdrv_write_str(ttd, "\e[D", 3);
   else if(rightward < -1)
     tickit_termdrv_write_strf(ttd, "\e[%dD", -rightward);
+
+  return true;
 }
 
-static int scrollrect(TickitTermDriver *ttd, const TickitRect *rect, int downward, int rightward)
+static bool scrollrect(TickitTermDriver *ttd, const TickitRect *rect, int downward, int rightward)
 {
   struct XTermDriver *xd = (struct XTermDriver *)ttd;
 
   if(!downward && !rightward)
-    return 1;
+    return true;
 
   int term_cols;
   tickit_term_get_size(ttd->tt, NULL, &term_cols);
@@ -111,7 +114,7 @@ static int scrollrect(TickitTermDriver *ttd, const TickitRect *rect, int downwar
     if(right < term_cols)
       tickit_termdrv_write_strf(ttd, "\e[s");
 
-    return 1;
+    return true;
   }
 
   if(xd->cap.slrm ||
@@ -146,16 +149,16 @@ static int scrollrect(TickitTermDriver *ttd, const TickitRect *rect, int downwar
     if(rect->left > 0 || right < term_cols)
       tickit_termdrv_write_str(ttd, "\e[s", 3);
 
-    return 1;
+    return true;
   }
 
-  return 0;
+  return false;
 }
 
-static void erasech(TickitTermDriver *ttd, int count, int moveend)
+static bool erasech(TickitTermDriver *ttd, int count, TickitMaybeBool moveend)
 {
   if(count < 1)
-    return;
+    return true;
 
   /* Only use ECH if we're not in reverse-video mode. xterm doesn't do rv+ECH
    * properly
@@ -166,7 +169,7 @@ static void erasech(TickitTermDriver *ttd, int count, int moveend)
     else
       tickit_termdrv_write_strf(ttd, "\e[%dX", count);
 
-    if(moveend == 1)
+    if(moveend == TICKIT_YES)
       move_rel(ttd, 0, count);
   }
   else {
@@ -180,14 +183,17 @@ static void erasech(TickitTermDriver *ttd, int count, int moveend)
     }
     tickit_termdrv_write_str(ttd, spaces, count);
 
-    if(moveend == 0)
+    if(moveend == TICKIT_NO)
       move_rel(ttd, 0, -count);
   }
+
+  return true;
 }
 
-static void clear(TickitTermDriver *ttd)
+static bool clear(TickitTermDriver *ttd)
 {
   tickit_termdrv_write_strf(ttd, "\e[2J", 4);
+  return true;
 }
 
 static struct SgrOnOff { int on, off; } sgr_onoff[] = {
@@ -202,7 +208,7 @@ static struct SgrOnOff { int on, off; } sgr_onoff[] = {
   {  5, 25 }, /* blink */
 };
 
-static void chpen(TickitTermDriver *ttd, const TickitPen *delta, const TickitPen *final)
+static bool chpen(TickitTermDriver *ttd, const TickitPen *delta, const TickitPen *final)
 {
   /* There can be at most 12 SGR parameters; 3 from each of 2 colours, and
    * 6 single attributes
@@ -259,7 +265,7 @@ static void chpen(TickitTermDriver *ttd, const TickitPen *delta, const TickitPen
   }
 
   if(pindex == 0)
-    return;
+    return true;
 
   /* If we're going to clear all the attributes then empty SGR is neater */
   if(!tickit_pen_is_nondefault(final))
@@ -285,43 +291,45 @@ static void chpen(TickitTermDriver *ttd, const TickitPen *delta, const TickitPen
   sprintf(s, "m");
 
   tickit_termdrv_write_str(ttd, buffer, len);
+
+  return true;
 }
 
-static int getctl_int(TickitTermDriver *ttd, TickitTermCtl ctl, int *value)
+static bool getctl_int(TickitTermDriver *ttd, TickitTermCtl ctl, int *value)
 {
   struct XTermDriver *xd = (struct XTermDriver *)ttd;
 
   switch(ctl) {
     case TICKIT_TERMCTL_ALTSCREEN:
       *value = xd->mode.altscreen;
-      return 1;
+      return true;
 
     case TICKIT_TERMCTL_CURSORVIS:
       *value = xd->mode.cursorvis;
-      return 1;
+      return true;
 
     case TICKIT_TERMCTL_CURSORBLINK:
       *value = xd->mode.cursorblink;
-      return 1;
+      return true;
 
     case TICKIT_TERMCTL_MOUSE:
       *value = xd->mode.mouse;
-      return 1;
+      return true;
 
     case TICKIT_TERMCTL_CURSORSHAPE:
       *value = xd->mode.cursorshape;
-      return 1;
+      return true;
 
     case TICKIT_TERMCTL_KEYPAD_APP:
       *value = xd->mode.keypad;
-      return 1;
+      return true;
 
     case TICKIT_TERMCTL_COLORS:
       *value = 256;
-      return 1;
+      return true;
 
     default:
-      return 0;
+      return false;
   }
 }
 
@@ -338,38 +346,38 @@ static int mode_for_mouse(TickitTermMouseMode mode)
   return 0;
 }
 
-static int setctl_int(TickitTermDriver *ttd, TickitTermCtl ctl, int value)
+static bool setctl_int(TickitTermDriver *ttd, TickitTermCtl ctl, int value)
 {
   struct XTermDriver *xd = (struct XTermDriver *)ttd;
 
   switch(ctl) {
     case TICKIT_TERMCTL_ALTSCREEN:
       if(!xd->mode.altscreen == !value)
-        return 1;
+        return true;
 
       tickit_termdrv_write_str(ttd, value ? "\e[?1049h" : "\e[?1049l", 0);
       xd->mode.altscreen = !!value;
-      return 1;
+      return true;
 
     case TICKIT_TERMCTL_CURSORVIS:
       if(!xd->mode.cursorvis == !value)
-        return 1;
+        return true;
 
       tickit_termdrv_write_str(ttd, value ? "\e[?25h" : "\e[?25l", 0);
       xd->mode.cursorvis = !!value;
-      return 1;
+      return true;
 
     case TICKIT_TERMCTL_CURSORBLINK:
       if(xd->initialised.cursorblink && !xd->mode.cursorblink == !value)
-        return 1;
+        return true;
 
       tickit_termdrv_write_str(ttd, value ? "\e[?12h" : "\e[?12l", 0);
       xd->mode.cursorblink = !!value;
-      return 1;
+      return true;
 
     case TICKIT_TERMCTL_MOUSE:
       if(xd->mode.mouse == value)
-        return 1;
+        return true;
 
       /* Modes 1000, 1002 and 1003 are mutually exclusive; enabling any one
        * disables the other two
@@ -380,46 +388,46 @@ static int setctl_int(TickitTermDriver *ttd, TickitTermCtl ctl, int value)
         tickit_termdrv_write_strf(ttd, "\e[?%dh\e[?1006h", mode_for_mouse(value));
 
       xd->mode.mouse = value;
-      return 1;
+      return true;
 
     case TICKIT_TERMCTL_CURSORSHAPE:
       if(xd->initialised.cursorshape && xd->mode.cursorshape == value)
-        return 1;
+        return true;
 
       if(xd->cap.cursorshape)
         tickit_termdrv_write_strf(ttd, "\e[%d q", value * 2 + (xd->mode.cursorblink ? -1 : 0));
       xd->mode.cursorshape = value;
-      return 1;
+      return true;
 
     case TICKIT_TERMCTL_KEYPAD_APP:
       if(!xd->mode.keypad == !value)
-        return 1;
+        return true;
 
       tickit_termdrv_write_strf(ttd, value ? "\e=" : "\e>");
-      return 1;
+      return true;
 
     default:
-      return 0;
+      return false;
   }
 }
 
-static int setctl_str(TickitTermDriver *ttd, TickitTermCtl ctl, const char *value)
+static bool setctl_str(TickitTermDriver *ttd, TickitTermCtl ctl, const char *value)
 {
   switch(ctl) {
     case TICKIT_TERMCTL_ICON_TEXT:
       tickit_termdrv_write_strf(ttd, "\e]1;%s\e\\", value);
-      return 1;
+      return true;
 
     case TICKIT_TERMCTL_TITLE_TEXT:
       tickit_termdrv_write_strf(ttd, "\e]2;%s\e\\", value);
-      return 1;
+      return true;
 
     case TICKIT_TERMCTL_ICONTITLE_TEXT:
       tickit_termdrv_write_strf(ttd, "\e]0;%s\e\\", value);
-      return 1;
+      return true;
 
     default:
-      return 0;
+      return false;
   }
 }
 
@@ -441,7 +449,7 @@ static void start(TickitTermDriver *ttd)
   tickit_termdrv_write_strf(ttd, "\e[G\e[K");
 }
 
-static int started(TickitTermDriver *ttd)
+static bool started(TickitTermDriver *ttd)
 {
   struct XTermDriver *xd = (struct XTermDriver *)ttd;
 
